package gf.core.main.utils;

import cnpc.cloud.auth.service.AuthCloudService;
import cnpc.cloud.auth.service.data.GetAuthBaseThirdAuthInput;
import cnpc.cloud.auth.service.data.GetAuthBaseThirdAuthOutput;
import xstartup.base.XContext;
import xstartup.base.XSelfServiceContext;
import xstartup.base.XSsoAuth;
import xstartup.base.XUser;
import xstartup.base.conf.XConfManager;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XUuidUtils;
import xstartup.core.base.provider.data.XToken;
import xstartup.data.XSingleResult;
import xstartup.feature.api.conf.XApiCookieConf;
import xstartup.helper.XTokenHelper;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

/**
 * @author lixueyan
 * @date 2023/4/7 0007 16:54
 */
public class SystemLoginUtil {
    private static ConcurrentMap<String,String> tokenMap = new ConcurrentHashMap();

    /**
     * 定时任务系统登陆
     */
    public static XContext systemLogin(XContext context, String tenantId){
        context = XSelfServiceContext.build(SystemLoginUtil.class);
        if(tokenMap.containsKey(tenantId)){
            XSingleResult<XUser> result = XTokenHelper.loadUser(context, tokenMap.get(tenantId));
            if(result.isSuccess()){
                context.setUser(result.getResult());
                return context;
            }else{
                tokenMap.remove(tenantId);
            }
        }

        AuthCloudService authService = context.getBean(AuthCloudService.class);
        GetAuthBaseThirdAuthInput input = new GetAuthBaseThirdAuthInput();
        input.setId(Integer.valueOf(tenantId));
        XSingleResult<GetAuthBaseThirdAuthOutput> result = authService.getTenantInfo(context, input);
        result.throwIfFail();
        GetAuthBaseThirdAuthOutput authOutput = result.getResult();
        if(authOutput==null){
            throw new XServiceException(999,"系统错误");
        }


        XApiCookieConf apiCookieConf = XConfManager.getConf(XApiCookieConf.class);
        Integer expiresIn = apiCookieConf.getExpires();

        String userName = authOutput.getTenantName(); //姓名
        Long orgId = 1L; //组织机构id
        Long roleId0 = 1L; //角色id1
        Long roleId1 = 1L; //角色id2
        Long userId = Long.valueOf(tenantId); //用户id
        String userCode = authOutput.getAppId();//这个是用户字符串类型的唯一标志
        String ssoToken = null; //sso系统授权的token
        String ssoRefreshToken = null; //sso系统授权的刷新token
        String uniqueId = XUuidUtils.randomUUID();
        XUser user = XUser.createUser(XUser.ENTRY_API,
                orgId,
                new Long[]{roleId0, roleId1},
                userId,
                userCode,
                authOutput.getSmsExtendedCode(), //userAccount 存储租户短信扩展码
                userName,
                new XSsoAuth(uniqueId, ssoToken, ssoRefreshToken));
        XSingleResult<XToken> saveUserResult = XTokenHelper.saveUser(context, user, expiresIn);
        saveUserResult.throwIfException();
        tokenMap.putIfAbsent(tenantId,saveUserResult.getResult().getToken());
        return context;
    }
}
